<?php

namespace Ems\portalGestor\Controller;

use Ems\portalGestor\Service\EscalaService;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Error;

class EscalaController {
    /** @var EscalaService */
    protected $escalaService;

    public function __construct(
        EscalaService $escalaService
    ) {
        $this->escalaService = $escalaService;
    }

    public function getEscalaTrab(
        Request\Filter $request,
        Response $response
    ) {
        try {
            $row = $request->getParameters();

            $nrOrg            = (int)    $row['NRORG'];
            $dtMesCompetencia = (string) ($row['DTMESCOMPETENCIA'] ?? '');

            $escalas = $this->escalaService->getEscalaTrab(
                $nrOrg, $dtMesCompetencia
            );

            $response->addDataSet(new DataSet('data', $escalas));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getTurnos(
        Request\Filter $request,
        Response $response
    ) {
        try {
            $row = $request->getParameters();

            $nrOrg      = (int) $row['NRORG'];
            $nrEscala   = (int) $row['NRESCALATRABM'];

            $turnos = $this->escalaService->getTurnos(
                $nrOrg, $nrEscala
            );

            $response->addDataSet(new DataSet('data', $turnos));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }
}
