<?php
namespace Ems\portalGestor\Controller;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Error;

use Ems\portalGestor\Service\SolicitacaoService;
use Ems\portalGestor\Util\ExpectedException;

class SolicitacaoController {
    /** @var EntityManager */
    protected $entityManager;

    /** @var SolicitacaoService */
    protected $solicitacaoService;

    public function __construct(
        EntityManager $entityManager,
        SolicitacaoService $solicitacaoService
    ) {
        $this->entityManager        = $entityManager;
        $this->solicitacaoService   = $solicitacaoService;
    }


    public function getSolicitacoesVinculosGestor(
        Request\Filter $request,
        Response       $response
    ) {
        try {
            $row = $request->getParameters();

            $nrOrg                  = (int) $row['NRORG'];
            $nrEstrutura            = (int) $row['NRESTRUTURAM'];
            $cdOperadorAprovante    =       $row['CDOPERADOR_APROVANTE'] ?? null;

            $solicitacoes = $this->solicitacaoService->getSolicitacoesVinculosGestor(
                $nrOrg,
                $nrEstrutura,
                $cdOperadorAprovante
            );

            $response->addDataSet(new DataSet('data', $solicitacoes));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function getItemSolicitacao(DTO\Request\Filter $request, Response $response) {
        try {
            $params = $request->getParameters();
            $nrOrg             = $params['NRORG'];
            $nrsolicitacao         = $params['NRSOLICITACAO'];

            $dataSet = $this->solicitacaoService->getItemSolicitacao($nrOrg, $nrsolicitacao);

            $response->addDataSet(new DataSet('data', $dataSet));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }
    
    public function getTiposolicitacao(DTO\Request\Filter $request, Response $response) {
        try {
            $params = $request->getParameters();
            $nrOrg             = $params['NRORG'];

            $dataSet = $this->solicitacaoService->getTiposolicitacao($nrOrg);

            $response->addDataSet(new DataSet('data', $dataSet));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getSolicitacaoFile(
        Request\Filter $request,
        Response $response
    ) {
        try {
            $row = $request->getParameters();

            $nrOrg = (int) $row['NRORG'];
            $nrSolicitacao = (int) $row['NRSOLICITACAO'];

            $file = $this->solicitacaoService->getSolicitacaoFile(
                $nrOrg,
                $nrSolicitacao
            );

            $response->addDataSet(new DataSet('data', $file));

        } catch (\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }

    public function getTpAvisoPrevio(DTO\Request\Filter $request, Response $response) {
        try {
            $params = $request->getParameters();
            $nrOrg             = $params['NRORG'];

            $dataSet = $this->solicitacaoService->getTpAvisoPrevio($nrOrg);

            $response->addDataSet(new DataSet('data', $dataSet));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getTpDemissao(DTO\Request\Filter $request, Response $response) {
        try {
            $params = $request->getParameters();
            $nrOrg             = $params['NRORG'];

            $dataSet = $this->solicitacaoService->getTpDemissao($nrOrg);

            $response->addDataSet(new DataSet('data', $dataSet));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getMotivoResc(DTO\Request\Filter $request, Response $response) {
        try {
            $params = $request->getParameters();
            $nrOrg             = $params['NRORG'];

            $dataSet = $this->solicitacaoService->getMotivoResc($nrOrg);

            $response->addDataSet(new DataSet('data', $dataSet));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }
    
    public function getMotivoOcupa(DTO\Request\Filter $request, Response $response) {
        try {
            $params = $request->getParameters();
            $nrOrg             = $params['NRORG'];

            $dataSet = $this->solicitacaoService->getMotivoOcupa($nrOrg);

            $response->addDataSet(new DataSet('data', $dataSet));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }
    
    public function getOcupacao(DTO\Request\Filter $request, Response $response) {
        try {
            $params = $request->getParameters();
            $nrOrg             = $params['NRORG'];

            $dataSet = $this->solicitacaoService->getOcupacao($nrOrg);

            $response->addDataSet(new DataSet('data', $dataSet));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getModalidadeSalarial(DTO\Request\Filter $request, Response $response) {
        try {
            $params = $request->getParameters();
            $nrOrg             = $params['NRORG'];

            $dataSet = $this->solicitacaoService->getModalidadeSalarial($nrOrg);

            $response->addDataSet(new DataSet('data', $dataSet));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }
    
    public function getTipoAlteraSal(DTO\Request\Filter $request, Response $response) {
        try {
            $params = $request->getParameters();
            $nrOrg             = $params['NRORG'];

            $dataSet = $this->solicitacaoService->getTipoAlteraSal($nrOrg);

            $response->addDataSet(new DataSet('data', $dataSet));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }
    
    public function getUltimoSalario(DTO\Request\Filter $request, Response $response) {
        try {
            $params = $request->getParameters();
            $nrOrg             = $params['NRORG'];
            $nrvinculo             = $params['NRVINCULOM'];

            $dataSet = $this->solicitacaoService->getUltimoSalario($nrOrg,$nrvinculo);

            $response->addDataSet(new DataSet('data', $dataSet));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function solicitaAlteracao(DTO\Request\Row $request, DTO\Response $response){
        try {
            $row = $request->getRow();
            
            $success = $this->solicitacaoService->solicitaAlteracao($row);
            $response->addDataSet(new DataSet('data', array('success' => $success)));
        } catch (\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }

    public function getTipoRelacionamento(
        Request\Filter $request,
        Response       $response
        ) {
        try {
            $row = $request->getParameters();

            $nrOrg     = (int) $row['NRORG'];

            $tiposRel = $this->solicitacaoService->getTipoRelacionamento($nrOrg);

            $response->addDataSet(new DataSet('data', $tiposRel));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getLinhaTransporteAtiva(
        Request\Filter $request,
        Response       $response
        ) {
        try {
            $row = $request->getParameters();

            $nrOrg = (int) $row['NRORG'];

            $linhas = $this->solicitacaoService->getLinhaTransporteAtiva($nrOrg);

            $response->addDataSet(new DataSet('data', $linhas));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getTipolinhavinc(
        Request\Filter $request,
        Response       $response
        ) {
        try {
            $row = $request->getParameters();

            $nrOrg = (int) $row['NRORG'];

            $tiposLinhas = $this->solicitacaoService->getTipolinhavinc($nrOrg);

            $response->addDataSet(new DataSet('data', $tiposLinhas));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getIntegtransp(
        Request\Filter $request,
        Response       $response
        ) {
        try {
            $row = $request->getParameters();

            $nrOrg = (int) $row['NRORG'];

            $numerosIntegracao = $this->solicitacaoService->getIntegtransp($nrOrg);

            $response->addDataSet(new DataSet('data', $numerosIntegracao));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getSentidolinha(
        Request\Filter $request,
        Response       $response
        ) {
        try {
            $row = $request->getParameters();

            $nrOrg = (int) $row['NRORG'];

            $sentidos = $this->solicitacaoService->getSentidolinha($nrOrg);

            $response->addDataSet(new DataSet('data', $sentidos));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getBeneficio(
        Request\Filter $request,
        Response       $response
        ) {
        try {
            $row = $request->getParameters();

            $nrOrg = (int) $row['NRORG'];

            $beneficios = $this->solicitacaoService->getBeneficio($nrOrg);

            $response->addDataSet(new DataSet('data', $beneficios));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getPessoaRelacionamentos(
        Request\Filter $request,
        Response       $response
        ) {
        try {
            $row = $request->getParameters();

            $nrOrg =    (int) $row['NRORG'];
            $nrPessoa = (int) $row['NRPESSOA'];

            $pessoas = $this->solicitacaoService->getPessoaRelacionamentos(
                $nrOrg, $nrPessoa
            );

            $response->addDataSet(new DataSet('data', $pessoas));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getTipoDependente(
        Request\Filter $request,
        Response       $response
        ) {
        try {
            $row = $request->getParameters();

            $nrOrg =    (int) $row['NRORG'];

            $tiposDep = $this->solicitacaoService->getTipoDependente(
                $nrOrg
            );

            $response->addDataSet(new DataSet('data', $tiposDep));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getParcnegocioVinculos(
        Request\Filter $request,
        Response       $response
        ) {
        try {
            $row = $request->getParameters();

            $nrOrg =    (int) $row['NRORG'];

            $parcs = $this->solicitacaoService->getParcnegocioVinculos(
                $nrOrg
            );

            $response->addDataSet(new DataSet('data', $parcs));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getTipoprocessos(
        Request\Filter $request,
        Response       $response
        ) {
        try {
            $row = $request->getParameters();

            $nrOrg =    (int) $row['NRORG'];

            $tipos = $this->solicitacaoService->getTipoprocessos(
                $nrOrg
            );

            $response->addDataSet(new DataSet('data', $tipos));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getTemplate(
        Request\Filter $request,
        Response       $response
        ) {
        try {
            $row = $request->getParameters();

            $nrOrg =    (int) $row['NRORG'];

            $templates = $this->solicitacaoService->getTemplate(
                $nrOrg
            );

            $response->addDataSet(new DataSet('data', $templates));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getTipoTransferencia(
        Request\Filter $request,
        Response       $response
        ) {
        try {
            $row = $request->getParameters();

            $nrOrg =    (int) $row['NRORG'];

            $tipos = $this->solicitacaoService->getTipoTransferencia(
                $nrOrg
            );

            $response->addDataSet(new DataSet('data', $tipos));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getMovimentacaoTransferencia(
        Request\Filter $request,
        Response       $response
        ) {
        try {
            $row = $request->getParameters();

            $nrOrg =    (int) $row['NRORG'];

            $transferencias = $this->solicitacaoService->getMovimentacaoTransferencia(
                $nrOrg
            );

            $response->addDataSet(new DataSet('data', $transferencias));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getMotivoTransferencia(
        Request\Filter $request,
        Response       $response
        ) {
        try {
            $row = $request->getParameters();

            $nrOrg =    (int) $row['NRORG'];

            $motivos = $this->solicitacaoService->getMotivoTransferencia(
                $nrOrg
            );

            $response->addDataSet(new DataSet('data', $motivos));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function aprovaSolicitacao(
        Request\Row $request,
        Response    $response
        ) {
        try {
            $row = $request->getRow();

            $nrOrg                  =    (int)  $row['NRORG'];
            $nrSolicitacao          = (string)  $row['NRSOLICITACAO'];
            $cdOperadorAprovante    = (string)  $row['CDOPERADOR_APROVANTE'];
            $dsObservacao           =           $row['DSOBSERVACAO']            ?? null;
            $extraRow               =  (array)  $row['EXTRA_ROW']               ?? [];

            $aprovacao = $this->solicitacaoService->aprovaSolicitacao(
                $nrOrg,
                $nrSolicitacao,
                $cdOperadorAprovante,
                $dsObservacao,
                $extraRow
            );

            $response->addDataSet(new DataSet('data', $aprovacao));
        } catch (ExpectedException $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function reprovaSolicitacao(
        Request\Row $request,
        Response    $response
        ) {
        try {
            $row = $request->getRow();

            $nrOrg                  =    (int)  $row['NRORG'];
            $nrSolicitacao          = (string)  $row['NRSOLICITACAO'];
            $cdOperadorAprovante    = (string)  $row['CDOPERADOR_APROVANTE'];
            $dsObservacao           =           $row['DSOBSERVACAO']            ?? null;
            $extraRow               =  (array)  $row['EXTRA_ROW']               ?? [];

            $aprovacao = $this->solicitacaoService->reprovaSolicitacao(
                $nrOrg,
                $nrSolicitacao,
                $cdOperadorAprovante,
                $dsObservacao,
                $extraRow
            );

            $response->addDataSet(new DataSet('data', $aprovacao));
        } catch (ExpectedException $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }
}